/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.highlevel.History
 *
 *
 */
ML.load("xml/xmldomparser.js");

/**
 * HLHistory
 *   Message history implementation
 *
 * Constructor:
 *   HLHistory(jabberService)
 *
 * Methods:
 *   addObserver(observer, filter, sendIinitialy)
 *   removeObserver(observer)
 *   getHistoryItems(shortJid)
 *
 */

function HLHistory(jabberService)
{
    var file;
    var nodes, node;

    file = new File(jabberService.settingsDirectory, "history");
    if (!file.exists)
        file.createDirectory(0700);

    this.historyDir = file.path;

    file = new File(jabberService.settingsDirectory, "history.rdf");

    this.jabberService = jabberService;
    this.rdf = new FileRDF(file, JIM_NAMESPACE_URI+"rdf/history#", "hist");
    this.historyFiles = { };

    nodes = this.rdf.getContainer("rdf:jim-history-files", "seq").getElements();
    while (nodes.hasNext()) {
        node = nodes.getNext();

	this.historyFiles[node.getTarget("jid").value] =
	    node.getTarget("file").value;
    }

    this.observers = [];
    this.messages = [];
    jabberService.core.addObserver(this, 0);
}

HLHistory.prototype = {
    storeItem: function(jid, tag)
    {
	var shortJid, i;

	shortJid = this.jabberService.core.highlevel.
	    users.getShortJID(jid);

	this.messages.push(tag);
	this.storeItemOnDisc(shortJid, tag);

	for (i = 0; i < this.observers.length; i++)
	    if ((this.observers[i][1].flags & HLHistoryFilter.prototype.FLAG_JID) &&
		    (jid == this.observers[i][1].jid ||
		     shortJid == this.observers[i][1].jid))
		this.observers[i][0].onHLHistoryMessage(jid, tag);
    },

    storeItemOnDisc: function storeItemOnDisc(shortJid, tag)
    {
	var file, i, delays, date, offline;

	tag = tag.cloneNode(true);

	if (delays = tag.getElementsByTagNameNS("jabber:x:delay", "x")) {
	    for (i = 0; i < delays.length; i++) {
		if (domToString(delays[i]) == "Offline Storage")
		    date = utcStringToDate(delays[i].getAttribute("stamp"));
		delays[i].parentNode.removeChild(delays[i]);
	    }
	}
	tag.addElement("jabber:x:delay", "x",
			 "stamp", dateToUTCString(date || new Date())).
	    addText("Offline Local Storage");

	if (!this.historyFiles[shortJid]) {
	    var res = this.rdf.getResource();

	    do {
		file = new File(this.historyDir, generateRandomName() + ".xml");
	    } while (file.exists);

	    this.rdf.getContainer("rdf:jim-history-files", "seq").appendElement(res);
	    res.setAssertion("jid", shortJid, true);
	    res.setAssertion("file", file.path, true);
	    this.historyFiles[shortJid] = file.path;
	} else
	    file = new File(this.historyFiles[shortJid]);

	file.open(file.MODE_WRONLY|file.MODE_APPEND|file.MODE_CREATE, "UTF-8", 0600);
	file.write(domToString(tag)+"\n");
	file.close();
    },

    getMessageDate: function(tag)
    {
        var msg, delays, i;

	if (delays = tag.getElementsByTagNameNS("jabber:x:delay", "x"))
	    for (i = 0; i < delays.length; i++) {
                msg = domToText(delays[i]);
                if (msg == "Offline Storage" || msg == "Offline Local Storage")
                    return utcStringToDate(delays[i].getAttribute("stamp"));
	    }

        return new Date();
    },

    onMessage: function onMessage(tag)
    {
	this.storeItem(tag.getAttribute("from"), tag);
    },

    onMessageSend: function onMessageSend(tag)
    {
	this.storeItem(tag.getAttribute("to"), tag);
    },

    addObserver: function addObserver(observer, filter, sendIinitialy)
    {
	var content, messages;
	var shortJid, date, i, delay, msgDate;

	if (!observer.onHLHistoryMessage)
	    return;

	this.observers.push([observer, filter]);

	if (sendIinitialy) {
	    if (filter.flags & filter.FLAG_JID) {
		shortJid = this.jabberService.core.highlevel.
		    users.getShortJID(filter.jid);

		messages = this.getHistoryItems(shortJid);

		if (!messages)
		    return;

		if (filter.flags & filter.FLAG_TIME)
		    date = filter.time;

		for (i = 0; i < messages.length; i++) {
		    content = messages[i];

                    msgDate = this.getMessageDate(content).valueOf();

		    if (date && msgDate < date)
			continue;

		    if (filter.jid != shortJid &&
			    content.getAttribute("from") != filter.jid &&
			    content.getAttribute("to") != filter.jid)
			continue;

		    if ((filter.flags & filter.FLAG_OFFLINETIME) &&
                        msgDate > filter.offlineTime) {
                        content.removeChild(content.
                                            getElementsByTagNameNS("jabber:x:delay",
                                                                   "x").item(0));
                    }
		    observer.onHLHistoryMessage(filter.jid, content);
		}
	    } else {
		if (filter.flags & filter.FLAG_TIME)
		    date = flag.time;

		for (i = 0; i < this.messages.length; i++) {
		    content = this.messages[i];

		    if (date) {
			if (this.getMessageDate(content).valueOf() < date)
			    continue;
		    }
		    observer.onHLHistoryMessage(filter.jid, content);
		}
	    }
	}
    },

    removeObserver: function removeObserver(observer) {
        var i;

        for (i = 0; i < this.observers.length; i++)
            if (this.observers[i][0] == observer) {
                this.observers.splice(i, 1);
                break;
            }
    },

    getHistoryItems: function getHistoryItems(shortJid)
    {
	var file, items, content;

	if (this.historyFiles[shortJid]) {
	    file = new File(this.historyFiles[shortJid]);

            try {
                file.open(file.MODE_RDONLY, "UTF-8");
                content = "<hist xmlns='jabber:client'>"+file.read()+"</hist>";
                file.close();
            } catch (ex) {
                return null;
            }

	    items = [];
	    content = XMLDOMParser.prototype.parseFullyString(content).firstChild;

	    while (content) {
		if (content.nodeType == content.ELEMENT_NODE)
		    items.push(content);
		content = content.nextSibling;
	    }
	    return items;
	}
    },
}

function HLHistoryFilter(flags, jid, time, offlineTime) {
    this.flags = flags;

    if (jid)
	this.jid = jid;

    if (flags & this.FLAG_TIME) {
        time = time || getPref("days-in-history");
        this.time = new Date().valueOf() - time*24*60*60*1000;
    }

    if (flags & this.FLAG_OFFLINETIME)
        this.offlineTime = offlineTime.valueOf() - 1000;
}

HLHistoryFilter.prototype = {
    FLAG_JID: 1,
    FLAG_TIME: 2,
    FLAG_SESSION: 4,
    FLAG_OFFLINETIME: 8,

    flags: 0,
    jid: "",
    time: 0,
    offlineTime: 0,
}
