/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * utils.Exception
 *
 */

function ThrowException(message, reason) {
    var exc;

    exc = new Error(message,
		    Components.stack.caller.filename,
		    Components.stack.caller.lineNumber);
    if (reason)
	exc.reason = reason;

    throw exc;
}

function __exToStr__(exc, indent)
{
    var frames, stacktrace, pos, i, msg;

    if (exc)
	if (exc instanceof Components.interfaces.nsIException) {
	    stacktrace = "";
	    frames = exc.location;

	    while (frames && frames.caller) {
		stacktrace += indent+"  "+frames.name+"(<unknown>) at "+
		    (frames.filename||"<unknown>")+":"+
		    (frames.lineNumber||"<unknown>")+"\n";
		frames = frames.caller;
	    }
	    msg = indent+"Exception '"+exc.message+"' - "+
		exc.result+"("+exc.name+") thrown at " + 
		exc.filename+":"+exc.lineNumber+"\n";

	    if (exc.inner)
		msg += indent+"Caused by:\n" +
		    __exToStr__(exc.inner, indent+"  ");

	    return msg+indent+"Stacktrace:\n"+stacktrace;
	} else if (exc instanceof Error) {
	    frames = map (function(a) {
			      var m = a.lastIndexOf("@");
			      return [a.slice(0, m),a.slice(m+1)];
			  },
			  exc.stack.split("\n"));
	    stacktrace = "";
	    pos = exc.fileName+":"+exc.lineNumber;

	    for (i = 0; i < frames.length && frames[i][1] != pos; i++)
		;

	    for (; i < frames.length-2; i++)
		stacktrace += indent+"  "+frames[i][0] + " at "+frames[i][1]+"\n";

	    msg = indent+"Exception '"+exc.message+"' thrown at " + 
		exc.fileName+":"+exc.lineNumber +"\n";

	    if (exc.reason)
		msg += indent+"Caused by:\n" +
		    __exToStr__(exc.reason, indent+"  ");

	    return msg+indent+"Stacktrace:\n"+stacktrace;
	} else if (exc.code)
	    return indent+"Exception code: "+exc.code+"\n"+
		(exc.reason ? indent+"Caused by:\n" +
		 __exToStr__(exc.reason,indent+"  ")+"\n":"");
	else
	    return indent+exc.toSource();
    else
	return indent+exc;
}

Error.prototype.toString = function() {
    return __exToStr__(this, "");
}

Components.interfaces.nsIException.toString = Error.prototype.toString;
