/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * xml.XMLDomTraverse
 *
 *
 * Functions:
 *   xmlDOMTraverse(dom, actionTree, value, nodeName, [dontCheck])
 *   xmlDOMTraverseRec(dom, actionTree, value)
 *
 *   xmlDOMBuild(root, actionTree, value, nodeName, namespaceURI,
 *               dontCreateRootElement)
 *   xmlDOMBuildRec(root, actionTree, value)
 */
ML.load("xml/xmldombuilder.js");

function xmlDOMTraverse(dom, actionTree, val, nodeName, dontCheck)
{
    var action;

    if (!dom || !actionTree || !nodeName ||
	(!dontCheck && dom.localName != nodeName))
        return;

    if (action = actionTree["__PROCESS__"]) {
        if (typeof(action) == "function")
            action(dom, actionTree, val, nodeName);
        else if (action instanceof XMLDOMAction)
            action.handle(dom, actionTree, val, nodeName);
    } else
	xmlDOMTraverseRec(dom, actionTree, val);
}

function xmlDOMTraverseRec(dom, actionTree, val)
{
    var el, action, attrAction, i;

    if (action = actionTree["__ATTRIBUTES__"])
        if (typeof(action) == "function")
	    for (i = 0; i < dom.attributes.length; i++)
		action(dom, val, dom.attributes[i].nodeName,
		       dom.attributes[i].nodeValue);
        else if (action instanceof XMLDOMAttrAction)
	    for (i = 0; i < dom.attributes.length; i++)
		action.handle(dom, val, dom.attributes[i].nodeName,
			      dom.attributes[i].nodeValue);
	else if (typeof(action) == "object")
	    for (i = 0; i < dom.attributes.length; i++)
		if (attrAction = action[dom.attributes[i].nodeName])
		    if (typeof(attrAction) == "function")
			attrAction(dom, val, dom.attributes[i].nodeName,
				   dom.attributes[i].nodeValue);
		    else if (attrAction instanceof XMLDOMAttrAction)
			attrAction.handle(dom, val, dom.attributes[i].nodeName,
					  dom.attributes[i].nodeValue);
    
    el = dom.firstChild;
    while (el) {
        if (action = actionTree[el.localName])
            if (typeof(action) == "function")
                action(el, null, val, el.localName);
            else if (action instanceof XMLDOMAction)
                action.handle(el, null, val, el.localName);
            else
                xmlDOMTraverse(el, action, val, el.localName);
        el = el.nextSibling;
    }
}

function xmlDOMBuild(dom, actionTree, val, nodeName, namespaceURI, dontCreateNode)
{
    var action;

    if (!dom || !actionTree)
        return;

    if ((action = actionTree["__PROCESS__"]) && action instanceof XMLDOMAction)
	action.build(dom, actionTree, val, nodeName, namespaceURI);
    else if (action = actionTree["__BUILD_PROCESS__"]) {
        if (typeof(action) == "function")
            action(dom, actionTree, val, nodeName, namespaceURI);
	else if (action instanceof XMLDOMAction)
	    action.build(dom, actionTree, val, nodeName, namespaceURI);
    } else
	if (dontCreateNode)
	    xmlDOMBuildRec(dom, actionTree, val, namespaceURI);
	else
	    xmlDOMBuildRec(dom.addElement(namespaceURI, nodeName),
			   actionTree, val, namespaceURI);
}

function xmlDOMBuildRec(dom, actionTree, val, namespaceURI)
{
    var action, i;

    if (action = actionTree["__ATTRIBUTES__"])
        if (action instanceof XMLDOMAttrAction)
	    action.build(dom, val, null)
	else if (typeof(action) == "object")
	    for (i in action)
		if (action[i] instanceof XMLDOMAttrAction)
		    action[i].build(dom, val, i);

    for (i in actionTree)
	if (i != "__ATTRIBUTES__" &&
	    i != "__PROCESS__" &&
	    i != "__BUILD_PROCESS__")
	    {
		action = actionTree[i];
		if (action instanceof XMLDOMAction)
		    action.build(dom, null, val, i, namespaceURI);
		else if (typeof(action) == "object")
		    xmlDOMBuild(dom, action, val, i, namespaceURI);
	    }
}

function XMLDOMAction()
{
}

XMLDOMAction.prototype =
{
    handle: function(dom, actionTree, value, nodeName) { },
    build: function(dom, actionTree, value, nodeName, namespaceURI) { }
}

function XMLDOMAttrAction()
{
}

XMLDOMAttrAction.prototype =
{
    handle: function(dom, value, attrName, attrValue) { },
    build: function(dom, value, attrName) { }
}
